@@ -44,9 +44,10 @@ test_that("meme runs as expected", {
 })
 
 test_that("meme runs with added font family", {
-  sysname <- tolower(Sys.info()[["sysname"]]) # Skip on Solaris. Skip on Linux if not TRAVIS.
-  if(!sysname %in% c("windows", "mac", "linux") || (sysname == "linux" && !identical(Sys.getenv("TRAVIS"), "true")))
-    skip_on_os(os_skip)
+  a <- c("windows", "linux")
+  sysname <- tolower(Sys.info()[["sysname"]]) # Skip on Solaris, Mac. Skip on Linux if not TRAVIS.
+  skip_if(!sysname %in% a || (sysname == "linux" && !identical(Sys.getenv("TRAVIS"), "true")),
+          "Skipping test on current system.")
 
   fam <- "Arial"
   sysfonts::font_add(fam, "arial.ttf")
